#coding: utf-8

"""Offers "Log in via oDesk" functionality."""

O_REQUEST_TOKEN = '_ort'
"""Session key under which OAuth request token will be stored
during authentication."""

O_ACCESS_TOKEN = '_oat'
"""Session key under which OAuth access token will be stored
once authentication is complete."""
